from datetime import date
from com.sun.star.table import CellRangeAddress
from com.sun.star.sheet.GeneralFunction import MAX

import LeenoUtils
import SheetUtils
import LeenoSheetUtils
import LeenoComputo
import Dialogs
import LeenoDialogs as DLG
import pyleeno as PL
import LeenoEvents
import LeenoBasicBridge
import uno
import itertools
import operator


def sbloccaContabilita(oSheet, lrow):
    '''
    Controlla che non ci siano atti contabili registrati e dà il consenso a procedere.
    Ritorna True se il consenso è stato dato, False altrimenti
    '''
    if LeenoUtils.getGlobalVar('sblocca_computo') == 1:
        return True
    if oSheet.Name != 'CONTABILITA':
        return True

    partenza = LeenoSheetUtils.cercaPartenza(oSheet, lrow)
    if partenza[2] == '#reg':
        res = Dialogs.YesNoCancel(
           Title="Voce già registrata",
           Text= "Lavorando in questo punto del foglio,\n"
                 "comprometterai la validità degli atti contabili già emessi.\n\n"
                 "Vuoi procedere?\n\n"
                 "SCEGLIENDO SI' SARAI COSTRETTO A RIGENERARLI!"
        )
        if res == 1:
            LeenoUtils.setGlobalVar('sblocca_computo', 1)
            return True
        return False
    return True


# ###############################################################


def insertVoceContabilita(oSheet, lrow):
    '''
    Inserisce una nuova voce in CONTABILITA.
    '''
    # controllo che non ci siano atti registrati
    # se ci sono, chiede conferma per poter operare
    if not sbloccaContabilita(oSheet, lrow):
        return False

    stili_contab = LeenoUtils.getGlobalVar('stili_contab')
    stile = oSheet.getCellByPosition(0, lrow).CellStyle
    nSal = 0
    if stile == 'comp Int_colonna_R_prima':
        lrow += 1
    elif stile == 'Ultimus_centro_bordi_lati':
        i = lrow
        while i != 0:
            if oSheet.getCellByPosition(23, i).Value != 0:
                nSal = int(oSheet.getCellByPosition(23, i).Value)
                break
            i -= 1
        while oSheet.getCellByPosition(0, lrow).CellStyle == stile:
            lrow += 1
        if oSheet.getCellByPosition(0, lrow).CellStyle == 'uuuuu':
            lrow += 1
            #  nSal += 1
        #  else
    elif stile == 'Comp TOTALI':
        pass
    elif stile in stili_contab:
        sStRange = LeenoComputo.circoscriveVoceComputo(oSheet, lrow)
        nSal = int(oSheet.getCellByPosition(23, sStRange.RangeAddress.StartRow + 1).Value)
        lrow = LeenoSheetUtils.prossimaVoce(oSheet, lrow)
    else:
        return

    oDoc = SheetUtils.getDocumentFromSheet(oSheet)
    oSheetto = oDoc.getSheets().getByName('S5')
    oRangeAddress = oSheetto.getCellRangeByPosition(0, 22, 48, 26).getRangeAddress()
    oCellAddress = oSheet.getCellByPosition(0, lrow).getCellAddress()
    # inserisco le righe
    oSheet.getRows().insertByIndex(lrow, 5)
    oSheet.copyRange(oCellAddress, oRangeAddress)
    oSheet.getCellRangeByPosition(0, lrow, 48, lrow + 5).Rows.OptimalHeight = True

    # @@@ TO REMOVE !!!
    #_gotoCella(1, lrow + 1)

    sStRange = LeenoComputo.circoscriveVoceComputo(oSheet, lrow)
    sopra = sStRange.RangeAddress.StartRow
    for n in reversed(range(0, sopra)):
        if oSheet.getCellByPosition(1, n).CellStyle == 'Ultimus_centro_bordi_lati':
            break
        if oSheet.getCellByPosition(1, n).CellStyle == 'Data_bianca':
            data = oSheet.getCellByPosition(1, n).Value
            break
    try:
        oSheet.getCellByPosition(1, sopra + 2).Value = data
    except Exception:
        oSheet.getCellByPosition(1, sopra + 2).Value = date.today().toordinal() - 693594

    # raggruppo i righi di misura
    iSheet = oSheet.RangeAddress.Sheet
    oCellRangeAddr = CellRangeAddress()
    oCellRangeAddr.Sheet = iSheet
    oCellRangeAddr.StartColumn = 0
    oCellRangeAddr.EndColumn = 0
    oCellRangeAddr.StartRow = lrow + 2
    oCellRangeAddr.EndRow = lrow + 2
    oSheet.group(oCellRangeAddr, 1)
    ########################################################################

    if oDoc.NamedRanges.hasByName('_Lib_' + str(nSal)):
        if lrow - 1 == oSheet.getCellRangeByName('_Lib_' + str(nSal)).getRangeAddress().EndRow:
            nSal += 1

    oSheet.getCellByPosition(23, sopra + 1).Value = nSal
    oSheet.getCellByPosition(23, sopra + 1).CellStyle = 'Sal'

    oSheet.getCellByPosition(35, sopra + 4).Formula = '=B' + str(sopra + 2)
    oSheet.getCellByPosition(36, sopra +4).Formula = (
       '=IF(ISERROR(P' + str(sopra + 5) + ');"";IF(P' +
       str(sopra + 5) + '<>"";P' + str(sopra + 5) + ';""))')
    oSheet.getCellByPosition(36, sopra + 4).CellStyle = "comp -controolo"

    LeenoSheetUtils.numeraVoci(oSheet, 0, True)

    '''
        @@@@ NOTA BENE : QUESTA PARTE È PER L'USO INTERATTIVO
        VEDIAMO CHE FARNE IN SEGUITO
    if cfg.read('Generale', 'pesca_auto') == '1':
        if arg == 0:
            return
        pesca_cod()
    '''

# ###############################################################


def svuotaContabilita(oDoc):
    '''
    svuota_contabilita
    Ricrea il foglio di contabilità partendo da zero.
    '''
    for n in range(1, 20):
        if oDoc.NamedRanges.hasByName('_Lib_' + str(n)):
            oDoc.NamedRanges.removeByName('_Lib_' + str(n))
            oDoc.NamedRanges.removeByName('_SAL_' + str(n))
            oDoc.NamedRanges.removeByName('_Reg_' + str(n))
    for el in ('Registro', 'SAL', 'CONTABILITA'):
        if oDoc.Sheets.hasByName(el):
            oDoc.Sheets.removeByName(el)

    oDoc.Sheets.insertNewByName('CONTABILITA', 3)
    oSheet = oDoc.Sheets.getByName('CONTABILITA')

    SheetUtils.setTabColor(oSheet, 16757935)
    oSheet.getCellRangeByName('C1').String = 'CONTABILITA'
    oSheet.getCellRangeByName('C1').CellStyle = 'comp Int_colonna'
    oSheet.getCellRangeByName('C1').CellBackColor = 16757935
    oSheet.getCellByPosition(0, 2).String = 'N.'
    oSheet.getCellByPosition(1, 2).String = 'Articolo\nData'
    oSheet.getCellByPosition(2, 2).String = 'LAVORAZIONI\nO PROVVISTE'
    oSheet.getCellByPosition(5, 2).String = 'P.U.\nCoeff.'
    oSheet.getCellByPosition(6, 2).String = 'Lung.'
    oSheet.getCellByPosition(7, 2).String = 'Larg.'
    oSheet.getCellByPosition(8, 2).String = 'Alt.\nPeso'
    oSheet.getCellByPosition(9, 2).String = 'Quantità\nPositive'
    oSheet.getCellByPosition(11, 2).String = 'Quantità\nNegative'
    oSheet.getCellByPosition(13, 2).String = 'Prezzo\nunitario'
    oSheet.getCellByPosition(15, 2).String = 'Importi'
    oSheet.getCellByPosition(16, 2).String = 'Incidenza\nsul totale'
    oSheet.getCellByPosition(17, 2).String = 'Sicurezza\ninclusa'
    oSheet.getCellByPosition(18, 2).String = 'importo totale\nsenza errori'
    oSheet.getCellByPosition(19, 2).String = 'Lib.\nN.'
    oSheet.getCellByPosition(20, 2).String = 'Lib.\nP.'
    oSheet.getCellByPosition(22, 2).String = 'flag'
    oSheet.getCellByPosition(23, 2).String = 'SAL\nN.'
    oSheet.getCellByPosition(25, 2).String = 'Importi\nSAL parziali'
    oSheet.getCellByPosition(27, 2).String = 'Sicurezza\nunitaria'
    oSheet.getCellByPosition(28, 2).String = 'Materiali\ne Noli €'
    oSheet.getCellByPosition(29, 2).String = 'Incidenza\nMdO %'
    oSheet.getCellByPosition(30, 2).String = 'Importo\nMdO'
    oSheet.getCellByPosition(31, 2).String = 'Super Cat'
    oSheet.getCellByPosition(32, 2).String = 'Cat'
    oSheet.getCellByPosition(33, 2).String = 'Sub Cat'
    #  oSheet.getCellByPosition(34,2).String = 'tag B'sub Scrivi_header_moduli
    #  oSheet.getCellByPosition(35,2).String = 'tag C'
    oSheet.getCellByPosition(36, 2).String = 'Importi\nsenza errori'
    oSheet.getCellByPosition(0, 2).Rows.Height = 800
    #  colore colonne riga di intestazione
    oSheet.getCellRangeByPosition(0, 2, 36, 2).CellStyle = 'comp Int_colonna_R'
    oSheet.getCellByPosition(0, 2).CellStyle = 'comp Int_colonna_R_prima'
    oSheet.getCellByPosition(18, 2).CellStyle = 'COnt_noP'
    oSheet.getCellRangeByPosition(0, 0, 0, 3).Rows.OptimalHeight = True
    #  riga di controllo importo
    oSheet.getCellRangeByPosition(0, 1, 36, 1).CellStyle = 'comp In testa'
    oSheet.getCellByPosition(2, 1).String = 'QUESTA RIGA NON VIENE STAMPATA'
    oSheet.getCellRangeByPosition(0, 1, 1, 1).merge(True)
    oSheet.getCellByPosition(13, 1).String = 'TOTALE:'
    oSheet.getCellByPosition(20, 1).String = 'SAL SUCCESSIVO:'

    oSheet.getCellByPosition(25, 1).Formula = '=$P$2-SUBTOTAL(9;$P$2:$P$2)'

    oSheet.getCellByPosition(15,
                             1).Formula = '=SUBTOTAL(9;P3:P4)'  # importo lavori
    oSheet.getCellByPosition(0, 1).Formula = '=AK2'  # importo lavori
    oSheet.getCellByPosition(
        17, 1).Formula = '=SUBTOTAL(9;R3:R4)'  # importo sicurezza

    oSheet.getCellByPosition(
        28, 1).Formula = '=SUBTOTAL(9;AC3:AC4)'  # importo materiali
    oSheet.getCellByPosition(29,
                             1).Formula = '=AE2/Z2'  # Incidenza manodopera %
    oSheet.getCellByPosition(29, 1).CellStyle = 'Comp TOTALI %'
    oSheet.getCellByPosition(
        30, 1).Formula = '=SUBTOTAL(9;AE3:AE4)'  # importo manodopera
    oSheet.getCellByPosition(
        36, 1).Formula = '=SUBTOTAL(9;AK3:AK4)'  # importo certo

    # riga del totale
    oSheet.getCellByPosition(2, 3).String = 'T O T A L E'
    oSheet.getCellByPosition(15,
                             3).Formula = '=SUBTOTAL(9;P3:P4)'  # importo lavori
    oSheet.getCellByPosition(
        17, 3).Formula = '=SUBTOTAL(9;R3:R4)'  # importo sicurezza
    oSheet.getCellByPosition(
        30, 3).Formula = '=SUBTOTAL(9;AE3:AE4)'  # importo manodopera
    oSheet.getCellRangeByPosition(0, 3, 36, 3).CellStyle = 'Comp TOTALI'
    # riga rossa
    oSheet.getCellByPosition(0, 4).String = 'Fine Computo'
    oSheet.getCellRangeByPosition(0, 4, 36, 4).CellStyle = 'Riga_rossa_Chiudi'

    LeenoSheetUtils.setLarghezzaColonne(oSheet)

    return oSheet


# ###############################################################


def generaContabilita(oDoc):
    '''
    Mostra il foglio di contabilità, se presente
    Altrimenti lo genera
    '''
    if oDoc.Sheets.hasByName('S1'):
        oDoc.Sheets.getByName('S1').getCellByPosition(7, 327).Value = 1
        if oDoc.Sheets.hasByName('CONTABILITA'):
            oSheet = oDoc.Sheets.getByName('CONTABILITA')
        else:
            #oSheet = oDoc.Sheets.insertNewByName('CONTABILITA', 5)
            oSheet = svuotaContabilita(oDoc)
            insertVoceContabilita(oSheet, 0)

            LeenoEvents.assegna()
            LeenoSheetUtils.ScriviNomeDocumentoPrincipaleInFoglio(oSheet)

    return oSheet

########################################################################
# CONTABILITA ## CONTABILITA ## CONTABILITA ## CONTABILITA ## CONTABILITA #


def attiva_contabilita():
    '''
    Se presenti, attiva e visualizza le tabelle di contabilità
    @@@ MODIFICA IN CORSO CON 'LeenoContab.generaContabilita'
    '''
    PL.chiudi_dialoghi()
    oDoc = LeenoUtils.getDocument()
    if oDoc.Sheets.hasByName('S1'):
        oDoc.Sheets.getByName('S1').getCellByPosition(7, 327).Value = 1
        if oDoc.Sheets.hasByName('CONTABILITA'):
            for el in ('Registro', 'SAL', 'CONTABILITA'):
                if oDoc.Sheets.hasByName(el):
                    PL.GotoSheet(el)
        else:
            oDoc.Sheets.insertNewByName('CONTABILITA', 5)
            svuotaContabilita(oDoc)
            PL.GotoSheet('CONTABILITA')
            PL._gotoCella(0, 2)
        PL.GotoSheet('CONTABILITA')
    LeenoBasicBridge.ScriviNomeDocumentoPrincipale()
    LeenoEvents.assegna()
########################################################################


def partita(testo):
    '''
    Aggiunge/detrae rigo di PARTITA PROVVISORIA
    '''
    oDoc = LeenoUtils.getDocument()
    oSheet = oDoc.getSheets().getByName(oDoc.CurrentController.ActiveSheet.Name)
    if oSheet.Name != "CONTABILITA":
        return
    x = PL.LeggiPosizioneCorrente()[1]
    if oSheet.getCellByPosition(0, x).CellStyle == 'comp 10 s_R':
        if oSheet.getCellByPosition(2, x).Type.value != 'EMPTY':
            PL.Copia_riga_Ent()
            x += 1
        oSheet.getCellByPosition(2, x).String = testo
        oSheet.getCellRangeByPosition(2, x, 8, x).CellBackColor = 16777113
        PL._gotoCella(5, x)


def MENU_partita_aggiungi():
    '''
    @@ DA DOCUMENTARE
    '''
    partita('PARTITA PROVVISORIA')


def MENU_partita_detrai():
    '''
    @@ DA DOCUMENTARE
    '''
    partita('SI DETRAE PARTITA PROVVISORIA')


########################################################################
def struttura_CONTAB():
    '''
    Visualizza in modalità struttura i documenti contabili
    '''
    oDoc = LeenoUtils.getDocument()
    oSheet = oDoc.getSheets().getByName(oDoc.CurrentController.ActiveSheet.Name)
    PL.struttura_off()
    oRanges = oDoc.NamedRanges
    
    if oSheet.Name == 'CONTABILITA':
        pref = "_Lib_"
        y = 3
    elif oSheet.Name == 'Registro':
        pref = "_Reg_"
        y = 1
    elif oSheet.Name == 'SAL':
        pref = "_SAL_"
        y = 1

    for i in range(1, 50):
        try:
            oRange=oRanges.getByName(pref + str(i)).ReferredCells.RangeAddress
            # ~oSheet.ungroup(oRange, 1)
            oSheet.group(oRange, 1)
            oSheet.getCellRangeByPosition(0, oRange.StartRow,
                11, oRange.EndRow).Rows.IsVisible = False
        except:
            try:
                oSheet.getCellRangeByPosition(0, oRange.StartRow,
                    11, oRange.EndRow).Rows.IsVisible = True
                PL._gotoCella(0, oRange.StartRow -1)
                oDoc.CurrentController.setFirstVisibleRow(y)
            except:
                # ~Dialogs.NotifyDialog(Image='Icons-Big/info.png',
                        # ~Title = 'Info',
                        # ~Text='''In questo Libretto delle Misure
# ~non ci sono misure registrate.''')
                pass
            return

def GeneraLibretto(oDoc):
    '''
    CONTABILITA' - Si ottiene una riga gialla con l'indicazione delle
    voci di misurazione registrate ed un parziale dell'importo del SAL a
    cui segue la visualizzazione in struttura delle voci registrate nel
    Libretto delle Misure.
    '''

    # ~oDoc = LeenoUtils.getDocument()
    #  DLG.mri(oDoc.StyleFamilies.getByName("CellStyles").getByName('comp 1-a PU'))
    #  return
    oSheet = oDoc.getSheets().getByName(oDoc.CurrentController.ActiveSheet.Name)
    if oSheet.Name != 'CONTABILITA':
        return
    PL.numera_voci()
    oRanges = oDoc.NamedRanges

    # ~try:
        # ~oRanges.removeByName("_Lib_1")
    # ~except:
        # ~pass

    # ~return
    #trovo il numero del nuovo sal
    nSal = 0
    for i in reversed(range(1, 50)):
        if oRanges.hasByName("_Lib_" + str(i)) == True:
            nSal = i +1
            break
        else:
            nSal = 1
            daVoce = 1
            old_nPage = 1
    libretti = SheetUtils.sStrColtoList('segue Libretto delle Misure n.', 2, oSheet, start=2)
    try:
        daVoce = int(oSheet.getCellByPosition(2, libretti[-1]
        ).String.split('÷')[1]) + 1
    except:
        daVoce = 1
    oCellRange = oSheet.getCellRangeByPosition(0, 3, 0,
        SheetUtils.getUsedArea(oSheet).EndRow - 2)
    if daVoce >= int(oCellRange.computeFunction(MAX)):
        Dialogs.NotifyDialog(Image='Icons-Big/exclamation.png',
                Title = 'ATTENZIONE!',
                Text='Tutte le voci di questo Libretto delle Misure\n'
                    'sono già registrate.')
        return

    nomearea="_Lib_" + str(nSal)

    #  Recupero la prima riga non registrata

    daVoce = PL.InputBox(str(daVoce), "Registra Libretto, da voce n.")
    if len(daVoce) ==0:
        return

    try:
        lrow = int(SheetUtils.uFindStringCol(daVoce, 0, oSheet))
    except TypeError:
        return
    sStRange = LeenoComputo.circoscriveVoceComputo(oSheet, lrow)
    primariga = sStRange.RangeAddress.StartRow

    #  ULTIMA VOCE
    oCellRange = oSheet.getCellRangeByPosition(
        0, 3, 0, SheetUtils.getUsedArea(oSheet).EndRow - 2)
    aVoce = int(oCellRange.computeFunction(MAX))

    aVoce = PL.InputBox(str(aVoce), "Registra Libretto, a voce n.")
    # ~aVoce = str(int(daVoce) + 1)
    if len(aVoce) == 0:
        return
    elif int(aVoce) < int(aVoce):
        return

    try:
        lrow = int(SheetUtils.uFindStringCol(aVoce, 0, oSheet))
    except TypeError:
        return
    sStRange = LeenoComputo.circoscriveVoceComputo(oSheet, lrow)
    ultimariga = sStRange.RangeAddress.EndRow

    # attiva la progressbar
    progress = Dialogs.Progress(Title='Generazione elaborato...', Text="Libretto delle Misure")
    progress.setLimits(1, 6)
    progress.setValue(0)
    progress.show()
    progress.setValue(1)

    # Recupero i dati per il SAL
    # ottengo datiSAL = [art,  desc, um, quant] in cui quant è la "somma a tutto il"
    SAL = []
    i = 5
    while i < ultimariga + 1:
        '''
        SAL = (art,  desc, um, quant, prezzo, importo)
        EP = elenco articoli
        '''
        datiSAL = LeenoComputo.datiVoceComputo(oSheet, i)[1]
        SAL.append(datiSAL)
        i= LeenoSheetUtils.prossimaVoce(oSheet, i)
    sic = []
    mdo = []
    for el in SAL:
        sic.append(el[6])
        mdo.append(el[7])
    sic = sum(sic)
    mdo = sum(mdo)

    datiSAL=list()
    for k, g in itertools.groupby(sorted(SAL), operator.itemgetter(0,1,2)):
        quant = sum(float(q[3]) for q in g)
        k = list(k)
        k.append(quant)
        datiSAL.append(k)

    PL.comando ('DeletePrintArea')
    # ~oSheet.removeAllManualPageBreaks()
    SheetUtils.visualizza_PageBreak()

    oSheet.getCellByPosition(25, ultimariga - 1).String = "SAL n." + str(nSal)
    oSheet.getCellByPosition(25, ultimariga).Formula = (
        "=SUBTOTAL(9;P" + str(primariga + 1) + ":P" + str(ultimariga+1) + ")" )
    oSheet.getCellByPosition(25, ultimariga).CellStyle = "comp sotto Euro 3_R"

    # immetti le firme
    inizioFirme = ultimariga + 1

    PL.MENU_firme_in_calce (inizioFirme) # riga di inserimento
    fineFirme = inizioFirme + 10

    progress.setValue(2)
    area="$A$" + str(primariga + 1) + ":$AJ$" + str(fineFirme + 1)
    # ~LeenoBasicBridge.rifa_nomearea(oDoc, "CONTABILITA", area, nomearea)
    # ~DLG.chi(nomearea)

    SheetUtils.NominaArea(oDoc, "CONTABILITA", area, nomearea)



    oSheet.getCellRangeByPosition(0, inizioFirme, 32, fineFirme).CellStyle = "Ultimus_centro_bordi_lati"
    #applico gli stili corretti ad alcuni dati della firma
    oSheet.getCellByPosition(2, inizioFirme + 1).CellStyle = "Ultimus_destra"

    oNamedRange=oRanges.getByName(nomearea).ReferredCells.RangeAddress

    #range del _Lib_
    daRiga = oNamedRange.StartRow
    aRiga = oNamedRange.EndRow
    daColonna = oNamedRange.StartColumn
    aColonna = oNamedRange.EndColumn

    iSheet = oSheet.RangeAddress.Sheet
    # imposta area di stampa
    # ~oCellRangeAddr = uno.createUnoStruct('com.sun.star.table.CellRangeAddress')
    # ~oCellRangeAddr.Sheet = iSheet
    # ~oCellRangeAddr.StartColumn = daColonna
    # ~oCellRangeAddr.StartRow = daRiga
    # ~oCellRangeAddr.EndColumn = 11
    # ~oCellRangeAddr.EndRow = aRiga
    
    # imposta riga da ripetere
    oTitles = uno.createUnoStruct('com.sun.star.table.CellRangeAddress')
    oTitles.Sheet = iSheet
    oTitles.StartColumn = 0
    oTitles.StartRow = 2
    oTitles.EndColumn = 11
    oTitles.EndRow = 2
    oSheet.setTitleRows(oTitles)
    # ~oSheet.setPrintAreas((oCellRangeAddr,))
    oNamedRange.EndColumn = 11
    oSheet.setPrintAreas((oNamedRange,))
    oSheet.setPrintTitleRows(True)
    
    oSheet.PageStyle = "Page_Style_Libretto_Misure2" 

    progress.setValue(3)

    # sbianco l'area di stampa
    oSheet.getCellRangeByPosition(daColonna, daRiga, 11, aRiga).CellBackColor = -1

    progress.setValue(4)

    for i in range(0, 50):
        oSheet.getRows().insertByIndex(fineFirme, 1)
        oSheet.getCellByPosition(2, fineFirme).String = "===================="
        fineFirme += 1
        if oSheet.getCellByPosition(1, fineFirme).Rows.IsStartOfNewPage == True:
            fineFirme -= 1
            oSheet.getRows().removeByIndex(fineFirme, 1)
            break
    # ~oSheet.getRows().removeByIndex(fineFirme, 1)
    # ~fineFirme -=1

    oBordo = oSheet.getCellRangeByPosition(0, fineFirme, 32, fineFirme)
    bordo = oBordo.BottomBorder
    bordo.LineWidth = 2
    bordo.OuterLineWidth = 2
    oBordo.BottomBorder = bordo

    # ----------------------------------------------------------------------
    # QUESTA DEVE DIVENTARE UN'OPZIONE A SCELTA DELL'UTENTE
    # in caso di libretto unico questo if è da attivare
    # in modo che la numerazione delle pagine non ricominci da capo
    # ~if nSal > 1:
        # ~nLib = 1
    inumPag = 1 + old_nPage
    nLib = nSal

##########
    # COMPILO LA SITUAZIONE CONTABILE IN "S2" 1di2
    oS2 = oDoc.getSheets().getByName('S2')
    # trovo la posizione del titolo
    oEnd=SheetUtils.uFindString("SITUAZIONE CONTABILE", oS2)
    xS2=oEnd[1]
    yS2=oEnd[0]

    oS2.getCellByPosition(yS2 + nSal, xS2 + 1).Value = nSal
    oS2.getCellByPosition(yS2 + nSal, xS2 + 2).Value = date.today().toordinal() - 693594    #data
    oS2.getCellByPosition(yS2 + nSal, xS2 + 24).Value = aVoce        #ultima voce libretto
    oS2.getCellByPosition(yS2 + nSal, xS2 + 25).Value = inumPag      #ultima pagina libretto
##########

#  inumPag = 0'+ old_nPage 'SE IL LIBRETTO è UNICO

    #inserisco i dati
    progress.setValue(5)
    for i in range(primariga, fineFirme):
        if oSheet.getCellByPosition(1, i).CellStyle == "comp Art-EP_R":
            if primariga == 0:
                sStRange = LeenoComputo.circoscriveVoceComputo(oSheet, i)
                primariga = sStRange.RangeAddress.StartRow
            oSheet.getCellByPosition(19, i).Value= nLib     #numero libretto
            oSheet.getCellByPosition(22, i).String =  "#reg"  #flag registrato
            oSheet.getCellByPosition(23, i).Value= nSal     #numero SAL

            for nPag in range(0, len(oSheet.RowPageBreaks)):
                if i < oSheet.RowPageBreaks[nPag].Position:
                    oSheet.getCellByPosition(20, i).Value = nPag   #pagina
                    break

    progress.setValue(6)
    # annoto ultimo numero di pagina
    oSheet.getCellByPosition(20 , fineFirme).Value = nPag
    oSheet.getCellByPosition(20 , fineFirme).CellStyle = "num centro"
#  inumPag = nPag ' + old_nPage 'SE IL LIBRETTO è UNICO

    # ~SheetUtils.visualizza_PageBreak(False)

    # inserisco la prima riga GIALLA del LIBRETTO
    oSheet.getRows().insertByIndex(daRiga, 1)
    oSheet.getCellRangeByPosition (0, daRiga, 36, daRiga).CellStyle = "uuuuu"

    #range del _Lib_
    oSheet.getCellByPosition(2,  daRiga).String = (
        "segue Libretto delle Misure n." + str(nSal) +
        " - " + str(daVoce) + "÷" + str(aVoce)
        )
    oSheet.getCellByPosition(20, daRiga).Value =  nPag  #Pagina
    oSheet.getCellByPosition(19, daRiga).Value= nLib    #Libretto
    oSheet.getCellByPosition(23, daRiga).Value= nSal    #SAL
    oSheet.getCellByPosition(15, daRiga).Formula =(
        "=SUBTOTAL(9;$P$" + str(primariga + 2) + ":$P$" + str(
        ultimariga + 2) + ")"
        )
    oSheet.getCellByPosition(15, daRiga).CellStyle = "comp sotto Euro 3_R"
    oSheet.getCellByPosition(25, daRiga).Formula =(
        "=SUBTOTAL(9;$P$" + str(primariga + 2) + ":$P$" + str(
        ultimariga + 2) + ")"
        )
    oSheet.getCellByPosition(25, daRiga).CellStyle = "comp sotto Euro 3_R"

    # annoto il sal corrente sulla riga di intestazione
    oSheet.getCellByPosition(25, 2).Value = nSal
    oSheet.getCellByPosition(25, 2).CellStyle = "Menu_sfondo _input_grasBig"
    oSheet.getCellByPosition(25, 1).Formula = (
        "=$P$2-SUBTOTAL(9;$P$2:$P$" + str(ultimariga + 2) + ")"
        )

    PL._gotoCella(0, daRiga)
    # ~_gotoCella(0, inizioFirme)
    progress.hide()

#  Protezione_area ("CONTABILITA",nomearea)
#  Struttura_Contab ("_Lib_")
#  Genera_REGISTRO
    struttura_CONTAB()
    # ~for el in (nSal, daVoce, aVoce, primariga+1, ultimariga+1, datiSAL, sic, mdo):
        # ~DLG.chi(el)
    
    return nSal, daVoce, aVoce, primariga+1, ultimariga+1, datiSAL, sic, mdo


########################################################################
#  
#  name: GeneraRegistro
#  @param
#  @return
#  
def GeneraRegistro(oDoc):
    '''
    CONTABILITA' - genera un nuovo foglio 'Registro'. Si ottiene una riga
    gialla con l'indicazione delle voci di misurazione registrate ed un
    parziale dell'importo del SAL a cui segue la visualizzazione in
    struttura delle relative voci registrate nel Libretto delle Misure.
    '''

    progress = Dialogs.Progress(Title='Generazione elaborato...', Text="Registro di Contabilità")
    progress.setLimits(1, 8)
    progress.setValue(0)
    progress.show()
    progress.setValue(1)

    # ~zoom = oDoc.CurrentController.ZoomValue
    # ~oDoc.CurrentController.ZoomValue = 400
    # ~oDoc = LeenoUtils.getDocument()
    # ~Annulla_atto_contabile("Registro")
    # ~lista = GeneraLibretto(oDoc)
    # ~DLG.chi(lista)
    # ~return
    try:
        nSal, daVoce, aVoce, primariga, ultimariga, datiSAL, sic, mdo = GeneraLibretto(oDoc)
    except:
        return
        progress.hide()
    progress.setValue(2)

    # Recupero i dati per il Registro
    oSheet = oDoc.Sheets.getByName("CONTABILITA")
    REG = []
    i = primariga
    while i < ultimariga:
        '''
        REG = ((num + '\n' + art + '\n' + data), desc, Nlib, Plib, um,
            quantP, quantN, prezzo, importo)
        '''
        reg = LeenoComputo.datiVoceComputo(oSheet, i)[0]
        REG.append(reg)
        i= LeenoSheetUtils.prossimaVoce(oSheet, i)

    try:
        oDoc.getSheets().insertNewByName('Registro',5)
        PL.GotoSheet('Registro')
        oSheet = oDoc.Sheets.getByName('Registro')

    # riga di intestazione
        oSheet.getCellRangeByPosition(0,0,9,0).CellStyle="An.1v-Att Start"
        oSheet.getCellByPosition(0,0).String = ("N. ord.\nArticolo\nData")
        oSheet.getCellByPosition(1,0).String = ("LAVORAZIONI\nE SOMMINISTRAZIONI")
        oSheet.getCellByPosition(2,0).String = ("Lib.\nN.")
        oSheet.getCellByPosition(3,0).String = ("Lib.\nP.")
        oSheet.getCellByPosition(4,0).String = ("U.M.")
        oSheet.getCellByPosition(5,0).String = ("Quantità\nPositive")
        oSheet.getCellByPosition(6,0).String = ("Quantità\nNegative")
        oSheet.getCellByPosition(7,0).String = ("Prezzo\nunitario")
        oSheet.getCellByPosition(8,0).String = ("Importo\ndebito")
        oSheet.getCellByPosition(9,0).String = ("Importo\npagamento")
        # ~oSheet.getCellByPosition(10,0).String = ("Num.\nPag.")
    # larghezza colonne
        oSheet.getCellByPosition(0,0).Columns.Width = 1600 #'N. ord.
        oSheet.getCellByPosition(1,0).Columns.Width = 6600 #'LAVORAZIONI
        oSheet.getCellByPosition(2,0).Columns.Width = 650 #'Lib.N.
        oSheet.getCellByPosition(3,0).Columns.Width = 650 #'Lib.P.
        oSheet.getCellByPosition(4,0).Columns.Width = 1000 #'U.M.
        oSheet.getCellByPosition(5,0).Columns.Width = 1600 #'Positive
        oSheet.getCellByPosition(6,0).Columns.Width = 1600 #'Negative
        oSheet.getCellByPosition(7,0).Columns.Width = 1400 #'Prezzo
        oSheet.getCellByPosition(8,0).Columns.Width = 1950 #'debito
        oSheet.getCellByPosition(9,0).Columns.Width = 1950 #'pagamento
        # ~oSheet.getCellByPosition(0, 2).Rows.OptimalHeight = True
        # ~oSheet.getCellByPosition(10,0).Columns.OptimalWidth = True #'n.pag.
        insRow = 1 #'prima riga inserimento in Registro
    except:
        # recupera il registro precedente
        PL.GotoSheet('Registro')
        oSheet= oDoc.Sheets.getByName("Registro")
        # ~DLG.chi("_Reg_" + str(nSal - 1))
        oRanges = oDoc.NamedRanges
        oPrevRange = oRanges.getByName("_Reg_" + str(nSal - 1)).ReferredCells.RangeAddress

        fRow = oPrevRange.StartRow
        lRow = oPrevRange.EndRow
        insRow = oPrevRange.EndRow + 1
        
        # chiudo il registro precedente
        # ~oCell = oSheet.getCellRangeByPosition(0,fRow,11,lRow)
        # ~oCell.Rows.IsVisible=False

    progress.setValue(3)

    oSheet.PageStyle = 'PageStyle_REGISTRO_A4'

    # compilo il Registro
    lastRow = insRow + len(REG) -1
    oRange = oSheet.getCellRangeByPosition(0, insRow, 8, lastRow)
    # ~oDoc.CurrentController.select(oRange)
    reg = tuple(REG)
    # ~DLG.chi(insRow)
    # ~DLG.chi(len (reg[0]))
    oRange.setDataArray(reg)
    LeenoSheetUtils.adattaAltezzaRiga(oSheet)

    
# do gli stili al Registro
    oSheet.getCellRangeByPosition(0, insRow, 1, lastRow).CellStyle = "List-stringa-sin"
    oSheet.getCellRangeByPosition(2, insRow, 4, lastRow).CellStyle = "List-num-centro"
    oSheet.getCellRangeByPosition(5, insRow, 6, lastRow).CellStyle = "comp 1a"
    oSheet.getCellRangeByPosition(7, insRow, 9, lastRow).CellStyle = "List-num-euro"

# inserisco la prima riga GIALLA nel REGISTRO
    oSheet.getRows().insertByIndex(insRow, 1)
    oSheet.getCellRangeByPosition (0, insRow, 9, insRow).CellStyle = "uuuuu"
    PL.fissa()
    # ci metto le informazioni
    oSheet.getCellByPosition(1, insRow).String = "segue Registro n." + str(nSal) + " - " + str(daVoce) + "÷" + str(aVoce)
    oSheet.getCellByPosition(2, insRow).Value= nSal        #numero libretto
    oSheet.getCellByPosition(3, insRow).Value = REG[-1][3] #ultimo numero pagina
    # indico il parziale del SAL relativo:
    oSheet.getCellByPosition(8, insRow).Formula = (
        "=SUBTOTAL(9;I" + str(insRow +2) + ":I" + str(lastRow +2) + ")")
    oSheet.getCellByPosition(8, insRow).CellStyle = "comp sotto Euro 3_R"

    # RIGA RIPORTO
    insRow += 1
    oSheet.getRows().insertByIndex(insRow, 1)
    oSheet.getCellByPosition(1, insRow).String = "R I P O R T O"
    #debito
    oSheet.getCellByPosition(8, insRow).Formula = (
        '=IF(SUBTOTAL(9;$I$2:$I$' + str(insRow) + ')=0;"";SUBTOTAL(9;$I$2:$I$' + str(insRow))
    #pagamento
    oSheet.getCellByPosition(9, insRow).Formula = (
        '=IF(SUBTOTAL(9;$J$2:$J$' + str(insRow) + ')=0;"";SUBTOTAL(9;$J$2:$J$' + str(insRow))
        
    oSheet.getCellRangeByPosition (0, insRow, 9, insRow).CellStyle = "Ultimus_Bordo_sotto"

    insRow += 1
    oSheet.getRows().insertByIndex(insRow, 1)

    oSheet.getCellByPosition(1, insRow).String = "LAVORI A MISURA"
    oSheet.getCellRangeByPosition(0, insRow, 9, insRow).CellStyle = "Ultimus_centro_bordi_lati"
    PL._gotoCella(1, insRow)
    
    lastRow = insRow + len(REG)

    inizioFirme = lastRow + 5
    PL.MENU_firme_in_calce (inizioFirme) # riga di inserimento
    fineFirme = inizioFirme + 18

    progress.setValue(5)

# set area del REGISTRO
    area="$A$" + str(insRow) + ":$J$" + str(fineFirme + 1)
    nomearea = "_Reg_" + str(nSal)
    LeenoBasicBridge.rifa_nomearea(oDoc, "Registro", area , nomearea)

    oRanges = oDoc.NamedRanges
    oNamedRange=oRanges.getByName(nomearea).ReferredCells.RangeAddress

    #range del _Reg_
    # ~daRiga = oNamedRange.StartRow
    # ~aRiga = oNamedRange.EndRow
    # ~daColonna = oNamedRange.StartColumn
    # ~aColonna = oNamedRange.EndColumn

    iSheet = oSheet.RangeAddress.Sheet

    # imposta riga da ripetere
    oTitles = uno.createUnoStruct('com.sun.star.table.CellRangeAddress')
    oTitles.Sheet = iSheet
    # ~oTitles.StartColumn = 0
    oTitles.StartRow = 0
    # ~oTitles.EndColumn = 9
    # ~oTitles.EndRow = 0
    oSheet.setTitleRows(oTitles)
    # ~oSheet.setPrintAreas((oCellRangeAddr,))
    oSheet.setPrintAreas((oNamedRange,))
    oSheet.setPrintTitleRows(True)

    # ~oPrintArea = oSheet.getPrintAreas()
    # ~oSheet.group(oPrintArea[0], 1)

    oSheet.getCellRangeByPosition(0, lastRow +1, 9, fineFirme).CellStyle = "Ultimus_centro_bordi_lati"

    #torno su a completare...
    oSheet.getCellByPosition(1, lastRow + 2).String = "Parziale dei Lavori a Misura €"
    oSheet.getCellByPosition(1, lastRow + 2).CellStyle = "Ultimus_destra"
    oSheet.getCellByPosition(8, lastRow + 2).Formula = (
        '=SUBTOTAL(9;$I$2:$I$' + str(inizioFirme))
    oSheet.getCellByPosition(8, lastRow + 2).CellStyle = "Ultimus_destra_totali"
    
    oSheet.getCellByPosition(1, lastRow + 4).String = 'Lavori a tutto il ' + PL.oggi() + ' - T O T A L E   €'
    oSheet.getCellByPosition(1, lastRow + 4).CellStyle = "Ultimus_destra"
    oSheet.getCellByPosition(8, lastRow + 4).Formula = (
        '=SUBTOTAL(9;$I$2:$I$' + str(inizioFirme))
    oSheet.getCellByPosition(8, lastRow + 4).CellStyle = "Ultimus_destra_totali"

    #applico gli stili corretti ad alcuni dati della firma
    oSheet.getCellByPosition(1, lastRow + 6).CellStyle = "Ultimus_destra"
    oSheet.getCellByPosition(1, lastRow + 16).CellStyle = "Ultimus_destra"

    progress.setValue(6)

    LeenoSheetUtils.adattaAltezzaRiga(oSheet)
    for i in range(0, 50):
        oSheet.getRows().insertByIndex(fineFirme, 1)
        oSheet.getCellByPosition(1, fineFirme).String = "===================="
        fineFirme += 1
        if oSheet.getCellByPosition(1, fineFirme).Rows.IsStartOfNewPage == True:
            fineFirme -= 1
            oSheet.getRows().removeByIndex(fineFirme, 1)
            break
    # ~oSheet.getCellRangeByPosition(0, fineFirme, 9, fineFirme).CellStyle = "comp Descr"
    oSheet.getCellByPosition(1,fineFirme).String = "A   R I P O R T A R E"
    oSheet.getCellByPosition(8, fineFirme).Formula = (
        '=IF(SUBTOTAL(9;$I$2:$I$' + str(inizioFirme) + ')=0;"";SUBTOTAL(9;$I$2:$I$' + str(inizioFirme))
    oSheet.getCellByPosition(9, inizioFirme).Formula = (
        '=IF(SUBTOTAL(9;$J$2:$J$' + str(inizioFirme) + ')=0;"";SUBTOTAL(9;$J$2:$J$' + str(inizioFirme))
    oSheet.getCellRangeByPosition (0, fineFirme, 9, fineFirme).CellStyle = "Ultimus_Bordo_sotto"

    struttura_CONTAB()

    progress.setValue(6)
    progress.hide()

# ~def GeneraSAL (oDoc):

    progress = Dialogs.Progress(Title='Generazione elaborato...', Text="Registro di Contabilità")
    progress.setLimits(1, 6)
    progress.setValue(0)
    progress.show()
    progress.setValue(1)

    try:
        oDoc.getSheets().insertNewByName('SAL',6)
        PL.GotoSheet('SAL')
        oSheet = oDoc.Sheets.getByName('SAL')

    # riga di intestazione
        oSheet.getCellRangeByPosition(0,0,6,0).CellStyle="An.1v-Att Start"
        oSheet.getCellByPosition(0,0).String = ("N. ord.\nArticolo")
        oSheet.getCellByPosition(1,0).String = ("LAVORAZIONI\nE SOMMINISTRAZIONI")
        oSheet.getCellByPosition(2,0).String = ("U.M.")
        oSheet.getCellByPosition(3,0).String = ("Quantità")
        oSheet.getCellByPosition(4,0).String = ("Prezzo\nunitario")
        oSheet.getCellByPosition(5,0).String = ("Importo")
        oSheet.getCellByPosition(6,0).String = ("Pagine")
    # larghezza colonne
        oSheet.getCellByPosition(0,0).Columns.Width = 1600 #'N. ord.
        oSheet.getCellByPosition(1,0).Columns.Width = 10100 #'LAVORAZIONI
        oSheet.getCellByPosition(2,0).Columns.Width = 1500 #'U.M.
        oSheet.getCellByPosition(3,0).Columns.Width = 1800 #'Quantità
        oSheet.getCellByPosition(4,0).Columns.Width = 1400 #'Prezzo
        oSheet.getCellByPosition(5,0).Columns.Width = 1900 #'Importo
        oSheet.getCellByPosition(6,0).Columns.OptimalWidth = True #'n.pag.
        oSheet.getCellByPosition(0, 2).Rows.OptimalHeight = True
        insRow = 1 #'prima riga inserimento in Registro
    except:
        # recupera il registro precedente
        PL.GotoSheet('SAL')
        oSheet= oDoc.Sheets.getByName("SAL")
        # ~DLG.chi("_SAL_" + str(nSal - 1))
        oRanges = oDoc.NamedRanges
        oPrevRange = oRanges.getByName("_SAL_" + str(nSal - 1)).ReferredCells.RangeAddress

        fRow = oPrevRange.StartRow
        lRow = oPrevRange.EndRow
        insRow = oPrevRange.EndRow + 1

    oSheet.PageStyle = 'PageStyle_REGISTRO_A4'

    progress.setValue(2)

    # compilo il SAL
    lastRow = insRow + len(datiSAL) -1
    oRange = oSheet.getCellRangeByPosition(0, insRow, 3, lastRow)
    # ~oDoc.CurrentController.select(oRange)
    sal = tuple(datiSAL)
    # ~DLG.chi(insRow)
    # ~DLG.chi(len (sal[0]))
    oRange.setDataArray(sal)

    formule = []
    for x in range(insRow, lastRow + 1):
        formule.append(['=VLOOKUP(A' + str(x + 1) + ';elenco_prezzi;5;FALSE())',
            '=D' + str(x + 1) + '*E' + str(x + 1)]
            )

    progress.setValue(3)

# do gli stili al SAL
    oSheet.getCellRangeByPosition(0, insRow, 1, lastRow).CellStyle = "List-stringa-sin"
    oSheet.getCellRangeByPosition(2, insRow, 2, lastRow).CellStyle = "List-num-centro"
    oSheet.getCellRangeByPosition(3, insRow, 3, lastRow).CellStyle = "comp 1a"
    oSheet.getCellRangeByPosition(4, insRow, 5, lastRow).CellStyle = "List-num-euro"

# completo il SAL inserendo le formule
    oRange = oSheet.getCellRangeByPosition(4, insRow, 5, lastRow)
    formule = tuple(formule)
    oRange.setFormulaArray(formule)

    nOrd = 1
    for x in range(insRow, lastRow + 1):
        oSheet.getCellByPosition(4, x).Value = oSheet.getCellByPosition(4, x).Value
        oSheet.getCellByPosition(0, x).String = str(nOrd) \
            + '\n' + oSheet.getCellByPosition(0, x).String
        nOrd += 1
    LeenoSheetUtils.adattaAltezzaRiga(oSheet)

    progress.setValue(4)

# inserisco la prima riga GIALLA nel SAL
    oSheet.getRows().insertByIndex(insRow, 1)
    oSheet.getCellRangeByPosition (0, insRow, 9, insRow).CellStyle = "uuuuu"
    PL.fissa()
    # ci metto le informazioni
    # ~oSheet.getCellByPosition(1, insRow).String = "segue Stato di Avanzamento Lavori n." + str(nSal) + " - 1÷" + str(aVoce)
    oSheet.getCellByPosition(1, insRow).String = "segue Stato di Avanzamento Lavori n." + str(nSal) + " - " + str(daVoce) + "÷" + str(aVoce)
    # ~oSheet.getCellByPosition(2, insRow).Value = nSal        #numero libretto

    # parziale del SAL relativo:
    oSheet.getCellByPosition(5, insRow).Formula = (
        "=SUBTOTAL(9;$F$" + str(insRow +2) + ":F" + str(lastRow +2) + ")")
    oSheet.getCellByPosition(5, insRow).CellStyle = "comp sotto Euro 3_R"

    lastRow = insRow + len(datiSAL)

#torno su a completare...
    oSheet.getCellByPosition(1, lastRow + 2).String = "Parziale dei Lavori a Misura €"
    oSheet.getCellByPosition(5, lastRow + 2).Formula = (
        '=SUBTOTAL(9;$F$' + str(insRow) + ':$F$' + str(lastRow + 2))
    rigaPsal = lastRow + 2
    
    oSheet.getCellByPosition(1, lastRow + 4).String = 'Lavori a tutto il ' + PL.oggi() + ' - T O T A L E   €'
    oSheet.getCellByPosition(5, lastRow + 4).Formula = (
        '=SUBTOTAL(9;$F$' + str(insRow) + ':$F$' + str(lastRow + 2))

    progress.setValue(5)

    fineFirme = lastRow + 6
    for i in range(0, 50):
        oSheet.getRows().insertByIndex(fineFirme, 1)
        oSheet.getCellByPosition(1, fineFirme).String = "===================="
        fineFirme += 1
      


        if oSheet.getCellByPosition(1, fineFirme).Rows.IsStartOfNewPage == True:
            oDoc.CurrentController.select(oSheet.getCellByPosition(1, fineFirme))
            # ~ DLG.chi(oSheet.getCellByPosition(1, fineFirme).Rows.IsStartOfNewPage)
            # ~fineFirme -= 1
            # ~oSheet.getRows().removeByIndex(fineFirme, 1)
            break
    # ~return
#applico gli stili corretti ad alcuni dati della firma
    oSheet.getCellRangeByPosition(
        0, lastRow +1, 5, fineFirme).CellStyle = "Ultimus_centro_bordi_lati"
    oSheet.getCellRangeByPosition (0, fineFirme, 5, fineFirme).CellStyle = "comp Descr"
    oSheet.getCellByPosition(1, lastRow + 2).CellStyle = "Ultimus_destra"
    oSheet.getCellByPosition(1, lastRow + 4).CellStyle = "Ultimus_destra"
    oSheet.getCellByPosition(5, lastRow + 2).CellStyle = "Ultimus_destra_totali"
    oSheet.getCellByPosition(5, lastRow + 4).CellStyle = "Ultimus_destra_totali"

    lastRow = fineFirme + 1

    oSheet.getCellByPosition(1, lastRow + 1).String = "R I E P I L O G O   S A L"
     
    oSheet.getCellByPosition(1, lastRow + 3).String = ("Appalto: a misura")
    oSheet.getCellByPosition(1, lastRow + 4).String = ("Offerta: unico ribasso")

    oSheet.getCellByPosition(1, lastRow + 6).String = ("Lavori a Misura €")
    oSheet.getCellByPosition(5, lastRow + 6).Formula = "=$F$" + str(rigaPsal + 1)

    oSheet.getCellByPosition(1, lastRow + 7).String = ("Di cui importo per la Sicurezza")
    oSheet.getCellByPosition(5, lastRow + 7).Value = - sic

    oSheet.getCellByPosition(1, lastRow + 8).String = ("Di cui importo per la Manodopera")
    oSheet.getCellByPosition(5, lastRow + 8).Value = - mdo

    oSheet.getCellByPosition(1, lastRow + 9).String =  "Importo dei Lavori a Misura su cui applicare il ribasso"
    oSheet.getCellByPosition(5, lastRow + 9).Formula = "=SUM(F" + str(lastRow + 7) + ":F" + str(lastRow + 9) + ")"

    oSheet.getCellByPosition(1, lastRow + 10).Formula = (
    '''=CONCATENATE("RIBASSO del ";TEXT($S2.$C$78*100;"#.##0,00");"%")''')
    oSheet.getCellByPosition(5, lastRow + 10).Formula = "=-$F$" + str(lastRow + 10) + "*$S2.$C$78" # RIBASSO

    oSheet.getCellByPosition(1, lastRow + 11).String = ("Importo per la Sicurezza")
    oSheet.getCellByPosition(5, lastRow + 11).Value = sic

    oSheet.getCellByPosition(1, lastRow + 12).String = ("Importo per la Manodopera")
    oSheet.getCellByPosition(5, lastRow + 12).Value = mdo

    oSheet.getCellByPosition(1, lastRow + 13).String =  "PER I LAVORI A MISURA €"
    oSheet.getCellByPosition(5, lastRow + 13).Formula = "=SUM($F$" + str(lastRow + 10) + ":$F$" + str(lastRow + 13) + ")"

# IL TOTALE ANDRA' RISISTEMATO QUANDO SARANNO PRONTE LE ALTRE MODALITA' DI APPALTO: IN ECONOMIA E A CORPO
    oSheet.getCellByPosition(1, lastRow + 15).String =  "T O T A L E  €"
    oSheet.getCellByPosition(5, lastRow + 15).Formula = "=SUM($F$" + str(lastRow + 10) + ":$F$" + str(lastRow + 13) + ")"

    progress.setValue(6)

# ~# le firme
    inizioFirme = lastRow + 17
    PL.MENU_firme_in_calce (inizioFirme) # riga di inserimento
    fineFirme = inizioFirme + 12

    oSheet.getCellRangeByPosition(
        0, lastRow, 5, fineFirme).CellStyle = "Ultimus_centro_bordi_lati"
    oSheet.getCellByPosition(1, lastRow + 1).CellStyle = "Ultimus_centro_Dsottolineato"
    oSheet.getCellRangeByPosition(1, lastRow + 3, 1, lastRow + 4).CellStyle = "Ultimus_sx_italic"
    oSheet.getCellRangeByPosition (5, lastRow + 6,5, lastRow + 15).CellStyle = "ULTIMUS"
    oSheet.getCellByPosition(1, lastRow + 6).CellStyle = "Ultimus_sx_bold"
    oSheet.getCellRangeByPosition(1, lastRow + 7, 1, lastRow + 8).CellStyle = "Ultimus_sx"
    oSheet.getCellByPosition(5, lastRow + 8).CellStyle = "Ultimus_"
    oSheet.getCellRangeByPosition(1, lastRow + 9, 1, lastRow + 10).CellStyle = "Ultimus_destra"
    oSheet.getCellRangeByPosition(1, lastRow + 11, 1, lastRow + 12).CellStyle = "Ultimus_sx"
    oSheet.getCellByPosition(5, lastRow + 12).CellStyle = "Ultimus_"
    oSheet.getCellRangeByPosition(1, lastRow + 13, 1, lastRow + 13).CellStyle = "Ultimus_destra_bold"
    oSheet.getCellRangeByPosition(1, lastRow + 15, 1, lastRow + 15).CellStyle = "Ultimus_destra_bold"
    oSheet.getCellByPosition(5, lastRow + 15).CellStyle = "Ultimus_destra_totali"

    LeenoSheetUtils.adattaAltezzaRiga(oSheet)

    progress.setValue(7)

    # ~fineFirme = lastRow + 6
    for i in range(0, 50):
        oSheet.getRows().insertByIndex(fineFirme, 1)
        oSheet.getCellByPosition(1, fineFirme).String = "===================="
        fineFirme += 1
        if oSheet.getCellByPosition(1, fineFirme).Rows.IsStartOfNewPage == True:
            fineFirme -= 1
            oSheet.getRows().removeByIndex(fineFirme, 1)
            break

    oSheet.getCellRangeByPosition (0, fineFirme, 5, fineFirme).CellStyle = "comp Descr"

# set area di stampa del SAL
    area="$A$" + str(insRow + 2) + ":$F$" + str(fineFirme + 1)
    nomearea = "_SAL_" + str(nSal)
    LeenoBasicBridge.rifa_nomearea(oDoc, "SAL", area , nomearea)

    oRanges = oDoc.NamedRanges
    oNamedRange=oRanges.getByName(nomearea).ReferredCells.RangeAddress

# imposta riga da ripetere con area di stampa
    iSheet = oSheet.RangeAddress.Sheet
    oTitles = uno.createUnoStruct('com.sun.star.table.CellRangeAddress')
    oTitles.Sheet = iSheet
    oTitles.StartRow = 0
    oSheet.setTitleRows(oTitles)
    oSheet.setPrintAreas((oNamedRange,))
    oSheet.setPrintTitleRows(True)

    struttura_CONTAB()
    progress.setValue(8)
    progress.hide()
    # ~ThisComponent.CurrentController.setFirstVisibleRow(daRiga)
    # ~oDoc.CurrentController.ZoomValue = zoom
    return


########################################################################
def GeneraAttiContabili():
    '''
    @@ DA DOCUMENTARE
    '''
    oDoc = LeenoUtils.getDocument()
    oSheet = oDoc.getSheets().getByName(oDoc.CurrentController.ActiveSheet.Name)
    if oSheet.Name != "CONTABILITA":
        return
    if Dialogs.YesNoDialog(Title='Avviso',
        Text= 'Prima di procedere è consigliabile salvare il lavoro.\n\n'
            'Se continui, devi attendere il messaggio di procedura completata.\n'
            'Procedo senza salvare?') == 0:
        return
    # ~try:
        # ~nSal, daVoce, aVoce, fpRow, lpRow = GeneraLibretto(oDoc)
    # ~except:
        # ~return
    GeneraRegistro(oDoc)
    PL.GotoSheet('CONTABILITA')
    # ~DLG.chi((nSal, daVoce, aVoce, daRiga, aRiga))
    Dialogs.Info(Title = 'Voci registrate!',
        Text="La generazione degli allegati contabili è stata completata.\n"
            "Grazie per l'attesa.")


# CONTABILITA ## CONTABILITA ## CONTABILITA ## CONTABILITA ## CONTABILITA #
########################################################################
########################################################################
g_exportedScripts = GeneraAttiContabili
